/*     */ package org.eclipse.jetty.continuation;
/*     */ 
/*     */ import java.io.IOException;
/*     */ import java.util.ArrayList;
/*     */ import java.util.List;
/*     */ import javax.servlet.AsyncContext;
/*     */ import javax.servlet.AsyncEvent;
/*     */ import javax.servlet.AsyncListener;
/*     */ import javax.servlet.DispatcherType;
/*     */ import javax.servlet.ServletRequest;
/*     */ import javax.servlet.ServletResponse;
/*     */ import javax.servlet.ServletResponseWrapper;
/*     */ 
/*     */ public class Servlet3Continuation
/*     */   implements Continuation
/*     */ {
/*  26 */   private static final ContinuationThrowable __exception = new ContinuationThrowable();
/*     */   private final ServletRequest _request;
/*     */   private ServletResponse _response;
/*     */   private AsyncContext _context;
/*  31 */   private List<AsyncListener> _listeners = new ArrayList();
/*  32 */   private volatile boolean _initial = true;
/*  33 */   private volatile boolean _resumed = false;
/*  34 */   private volatile boolean _expired = false;
/*  35 */   private volatile boolean _responseWrapped = false;
/*     */ 
/*  37 */   private long _timeoutMs = -1L;
/*     */ 
/*     */   public Servlet3Continuation(ServletRequest request)
/*     */   {
/*  42 */     this._request = request;
/*     */ 
/*  44 */     this._listeners.add(new AsyncListener()
/*     */     {
/*     */       public void onComplete(AsyncEvent event) throws IOException
/*     */       {
/*     */       }
/*     */ 
/*     */       public void onError(AsyncEvent event) throws IOException
/*     */       {
/*     */       }
/*     */ 
/*     */       public void onStartAsync(AsyncEvent event) throws IOException
/*     */       {
/*  56 */         event.getAsyncContext().addListener(this);
/*     */       }
/*     */ 
/*     */       public void onTimeout(AsyncEvent event) throws IOException
/*     */       {
/*  61 */         Servlet3Continuation.this._initial = false;
/*  62 */         event.getAsyncContext().dispatch();
/*     */       }
/*     */     });
/*     */   }
/*     */ 
/*     */   public void addContinuationListener(final ContinuationListener listener)
/*     */   {
/*  70 */     AsyncListener wrapped = new AsyncListener()
/*     */     {
/*     */       public void onComplete(AsyncEvent event) throws IOException
/*     */       {
/*  74 */         listener.onComplete(Servlet3Continuation.this);
/*     */       }
/*     */ 
/*     */       public void onError(AsyncEvent event) throws IOException
/*     */       {
/*  79 */         listener.onComplete(Servlet3Continuation.this);
/*     */       }
/*     */ 
/*     */       public void onStartAsync(AsyncEvent event) throws IOException
/*     */       {
/*  84 */         event.getAsyncContext().addListener(this);
/*     */       }
/*     */ 
/*     */       public void onTimeout(AsyncEvent event) throws IOException
/*     */       {
/*  89 */         Servlet3Continuation.this._expired = true;
/*  90 */         listener.onTimeout(Servlet3Continuation.this);
/*     */       }
/*     */     };
/*  94 */     if (this._context != null)
/*  95 */       this._context.addListener(wrapped);
/*     */     else
/*  97 */       this._listeners.add(wrapped);
/*     */   }
/*     */ 
/*     */   public void complete()
/*     */   {
/* 103 */     AsyncContext context = this._context;
/* 104 */     if (context == null)
/* 105 */       throw new IllegalStateException();
/* 106 */     this._context.complete();
/*     */   }
/*     */ 
/*     */   public ServletResponse getServletResponse()
/*     */   {
/* 112 */     return this._response;
/*     */   }
/*     */ 
/*     */   public boolean isExpired()
/*     */   {
/* 118 */     return this._expired;
/*     */   }
/*     */ 
/*     */   public boolean isInitial()
/*     */   {
/* 125 */     return (this._initial) && (this._request.getDispatcherType() != DispatcherType.ASYNC);
/*     */   }
/*     */ 
/*     */   public boolean isResumed()
/*     */   {
/* 131 */     return this._resumed;
/*     */   }
/*     */ 
/*     */   public boolean isSuspended()
/*     */   {
/* 137 */     return this._request.isAsyncStarted();
/*     */   }
/*     */ 
/*     */   public void keepWrappers()
/*     */   {
/* 143 */     this._responseWrapped = true;
/*     */   }
/*     */ 
/*     */   public void resume()
/*     */   {
/* 149 */     AsyncContext context = this._context;
/* 150 */     if (context == null)
/* 151 */       throw new IllegalStateException();
/* 152 */     this._resumed = true;
/* 153 */     this._context.dispatch();
/*     */   }
/*     */ 
/*     */   public void setTimeout(long timeoutMs)
/*     */   {
/* 159 */     this._timeoutMs = timeoutMs;
/* 160 */     if (this._context != null)
/* 161 */       this._context.setTimeout(timeoutMs);
/*     */   }
/*     */ 
/*     */   public void suspend(ServletResponse response)
/*     */   {
/* 167 */     this._response = response;
/* 168 */     this._responseWrapped = (response instanceof ServletResponseWrapper);
/* 169 */     this._resumed = false;
/* 170 */     this._expired = false;
/* 171 */     this._context = this._request.startAsync();
/* 172 */     this._context.setTimeout(this._timeoutMs);
/*     */ 
/* 174 */     for (AsyncListener listener : this._listeners)
/* 175 */       this._context.addListener(listener);
/* 176 */     this._listeners.clear();
/*     */   }
/*     */ 
/*     */   public void suspend()
/*     */   {
/* 182 */     this._resumed = false;
/* 183 */     this._expired = false;
/* 184 */     this._context = this._request.startAsync();
/* 185 */     this._context.setTimeout(this._timeoutMs);
/*     */ 
/* 187 */     for (AsyncListener listener : this._listeners)
/* 188 */       this._context.addListener(listener);
/* 189 */     this._listeners.clear();
/*     */   }
/*     */ 
/*     */   public boolean isResponseWrapped()
/*     */   {
/* 195 */     return this._responseWrapped;
/*     */   }
/*     */ 
/*     */   public Object getAttribute(String name)
/*     */   {
/* 204 */     return this._request.getAttribute(name);
/*     */   }
/*     */ 
/*     */   public void removeAttribute(String name)
/*     */   {
/* 213 */     this._request.removeAttribute(name);
/*     */   }
/*     */ 
/*     */   public void setAttribute(String name, Object attribute)
/*     */   {
/* 222 */     this._request.setAttribute(name, attribute);
/*     */   }
/*     */ 
/*     */   public void undispatch()
/*     */   {
/* 231 */     if (isSuspended())
/*     */     {
/* 233 */       if (ContinuationFilter.__debug)
/* 234 */         throw new ContinuationThrowable();
/* 235 */       throw __exception;
/*     */     }
/* 237 */     throw new IllegalStateException("!suspended");
/*     */   }
/*     */ }

/* Location:           C:\Documents and Settings\anand.kumar\.m2\repository\org\eclipse\jetty\jetty-continuation\7.4.5.v20110725\jetty-continuation-7.4.5.v20110725.jar
 * Qualified Name:     org.eclipse.jetty.continuation.Servlet3Continuation
 * JD-Core Version:    0.6.1
 */